NOISE_PARTS = {
  'silence': parselmouth.Sound("silence_5minutes.wav"),
  'low': parselmouth.Sound("low_noise_5minutes.wav"),
  'high': parselmouth.Sound("high_noise_5minutes.wav")
}
START_SOUND = parselmouth.Sound("start.wav")
START_SILENCE_DURATION = 2 * 60
N_REPETITIONS = 3
TARGET_SAMPLING_FREQUENCY = 16000


def contains_consecutive_elements(a):
    return np.any(a[:-1] == a[1:])


def next_playback_order(rng):
    noise_part_names = list(NOISE_PARTS.keys())
    noise_part_idx = np.repeat(np.arange(len(noise_part_names)), N_REPETITIONS)
    while contains_consecutive_elements(noise_part_idx):
        rng.shuffle(noise_part_idx)
    return [noise_part_names[i] for i in noise_part_idx]


def create_next_playback(order):
    sf = START_SOUND.sampling_frequency
    extra_start_silence_duration = START_SILENCE_DURATION - START_SOUND.duration
    n_extra_start_silence_samples = int(sf * extra_start_silence_duration)
    extra_start_silence_samples = np.zeros(n_extra_start_silence_samples,
                                           dtype=float)
    extra_start_silence = parselmouth.Sound(extra_start_silence_samples, sf)
    sounds = [NOISE_PARTS[name] for name in order]
    concatenated = parselmouth.Sound.concatenate(
        [START_RECORDING_SOUND, extra_start_silence] + sounds)
    resampled = concatenated.resample(TARGET_SAMPLING_FREQUENCY)
    return resampled


rng = np.random.default_rng(42)
playback_orders = pd.DataFrame()
for i in range(N_PLAYBACKS):
    playback_order = next_playback_order(rng)
    playback_sound = create_next_playback(playback_order)
    playback_sound.save(f"playback_{i}.wav", 'WAV')

    playback_order_df = pd.DataFrame({
        'playback_name': f"playback_{i}",
        'noise_part_i': list(range(len(playback_order))),
        'noise_part': playback_order
    })
    playback_orders = playback_orders.append(playback_order_df, ignore_index=True)

playback_orders.to_csv("playback_orders.csv", index=False)
